"Use strict";
import powerbi from "powerbi-visuals-api";
import {ISelectorManager,ISelectorManagerConstructor, IFilterManager} from "../visual/visualInterfaces";
import * as d3 from "d3";
import * as settings from "../settings";
import * as selectors from "./selectors";
type Selection<T extends d3.BaseType> = d3.Selection<T, any, any, any>;
/*
负责页面上选择器的展示
*/
export class SelectorManager implements ISelectorManager{
    public selectorContainer : Selection<HTMLElement>;
    private selector:selectors.ISelector;
    public filterManager:IFilterManager;
    private categories: powerbi.DataViewCategoryColumn;
    private defaultSelect: powerbi.DataViewValueColumn;
    private defaultStart: powerbi.DataViewValueColumn;
    private defaultEnd: powerbi.DataViewValueColumn;
    constructor(selectorContainer:Selection<HTMLElement>,filterManager:IFilterManager){
        console.debug("selectorManager constructor start");
        this.selectorContainer=selectorContainer;
        this.filterManager=filterManager;
    }
    updateFormat(visualSettings:settings.VisualSettings,width:number,height:number): void {
        console.debug('selectorManager updateFormat start');
        console.debug('visualSettings:',visualSettings);
        if(visualSettings&&visualSettings.items){
            this.selector.updateFormat(visualSettings.items,width,height);
        }
        console.debug('selectorManager updateFormat end');

    }
    public dispose():void{
        this.selector.dispose();
        this.selectorContainer.remove();
    }
    public switchSelector<T extends selectors.ISelector>(classSelector:new ()=>T){
        let newSelector=new classSelector();
        this.selector?.dispose();
        this.selector=newSelector;
    }
    updateData(field: powerbi.DataViewCategoryColumn, defaultSelect: powerbi.DataViewValueColumn, defaultStart: powerbi.DataViewValueColumn, defaultEnd: powerbi.DataViewValueColumn){

        if(!this.selector){
            let fieldType=field.source.type;
            if(fieldType.text){
                let dropDownSelectorContainer=this.selectorContainer.append("div").classed("dropDown-selector-container",true);
                this.selector=new selectors.DropDownSelector(dropDownSelectorContainer,this.filterManager);
            }else if(fieldType.numeric){
                let dropDownSelectorContainer=this.selectorContainer.append("div").classed("dropDown-selector-container",true);
                this.selector=new selectors.DropDownSelector(dropDownSelectorContainer,this.filterManager);
            }else if(fieldType.dateTime){
                let dropDownSelectorContainer=this.selectorContainer.append("div").classed("dropDown-selector-container",true);
                this.selector=new selectors.DropDownSelector(dropDownSelectorContainer,this.filterManager);
            }else{
                throw "only receive text/num/datetime field";
            }
        }

        this.selector.update(field, defaultSelect, defaultStart, defaultEnd);
    };
}